using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;

using VirtualLab.Programming;
using VirtualLabAPI.Core.BasicFunctions;
using VirtualLabAPI.Core.Common;
using VirtualLabAPI.Core.DataVisualization;
using VirtualLabAPI.Core.FieldRepresentations;
using VirtualLabAPI.Core.Functions;
using VirtualLabAPI.Core.GeometryDescription;
using VirtualLabAPI.Core.LightPath;
using VirtualLabAPI.Core.Materials;
using VirtualLabAPI.Core.Modules;
using VirtualLabAPI.Core.Numerics;
using VirtualLabAPI.Core.OpticalSystems;
using VirtualLabAPI.Core.Propagation;

namespace OwnCode {
	/// <summary>
	/// this module can be used to visualize a surface selected from the user defined catalog
	/// the user can select whether to visualize the interface as 2D or 1D data array
	/// </summary>
	public class VLModule : IVLModule {
		/// <summary>
		/// the name of the interface within the user defined catalog which shall be visualized
		/// </summary>
		string surfaceNameInUserDefinedInterfaceCatalog = "TestSurface";

		/// <summary>
		/// boolean flag whether the surface should be visualized by a 1D line profile
		/// </summary>
		bool generateLineProfileOutput = false;

		/// <summary>
		/// the start position for the line profile
		/// </summary>
		VectorD startCoordinateLineProfile = new VectorD(-50e-6, 0);

		/// <summary>
		/// the start position for the line profile
		/// </summary>
		VectorD endCoordinateLineProfile = new VectorD(50e-6, 0);

		/// <summary>
		/// the number of data points along the line profile
		/// </summary>
		int numberOfPointForLineProfile = 10001;

		/// <summary>
		/// the size of the rectangle which shall be used for rectangular (2D) evaluation
		/// </summary>
		VectorD sizeForRectangleEvaluation = new VectorD(1.5e-3, 1.5e-3);

		/// <summary>
		/// the center position of the rectangle which shall be used for rectangular (2D) evaluation
		/// </summary>
		VectorD centerPositionRectangleEvaluation = new VectorD(0, 0);

		/// <summary>
		/// the number of sampling points  which shall be used for rectangular (2D) evaluation
		/// </summary>
		Vector numberSamplingPointRectangleEvaluation = new Vector(501, 501);

		/// <summary>
		/// run method of the module
		/// </summary>
		public void Run() {
			//load interface from user defined catalog
			InterfaceCatalog interfaceCatalog = InterfaceCatalog.LoadFromDefaultPath(CatalogDefinitionType.UserDefined, true);

			if (interfaceCatalog == null) {
				Globals.DataDisplay.LogError("User-defined catalog can not be opened!");
				return;
			}

			//select optical interface from user defined catalog
			OpticalInterface oInterface = interfaceCatalog[surfaceNameInUserDefinedInterfaceCatalog];

			//error handling
			if (oInterface == null) {
				Globals.DataDisplay.LogError("Interface can not be loaded.");
				return;
			}

			//convert optical interface into profile height interface
			ProfileHeightInterface pInterface = oInterface as ProfileHeightInterface;

			//check which kind of output shall be generated
			if (generateLineProfileOutput) {
				#region output 1D profile
				//calculate vector from start to end coordinate 
				VectorD vectorStartToEnd = endCoordinateLineProfile - startCoordinateLineProfile;
				//calculate lengths of profile line
				double lengthProfileLine = vectorStartToEnd.Abs();
				//normalize vector between start and end coordinate (used for calculation of phyiscal position along the profile line)
				vectorStartToEnd.Normalize();
				//calculate step width
				double stepWidthOnProfileLine = lengthProfileLine / (numberOfPointForLineProfile - 1);
				//generate real valued field to store height profile values
				ComplexFieldDerivative1D cfProfileLine = new ComplexFieldDerivative1D(numberOfPointForLineProfile, false, PrecisionMode.Double);
				//loop over all data points
				for (int runPoints = 0; runPoints < numberOfPointForLineProfile; runPoints++) {
					//calculate physical position
					VectorD currPosition = startCoordinateLineProfile + (runPoints * stepWidthOnProfileLine * vectorStartToEnd);
					//evaluate height profile at evaluate position
					double profileHeight = pInterface.ProfileHeight(currPosition);
					//store evaluated profile height within matrix
					cfProfileLine[runPoints] = profileHeight;
				}

				//generate 1D data array from given data
				DataArray1D daLineProfile = new DataArray1D(new ComplexField1DArray(new ComplexFieldDerivative1D[] { cfProfileLine }),
															new PhysicalProperty[] { PhysicalProperty.Length },
															new string[] { "Height" },
															stepWidthOnProfileLine,
															startCoordinateLineProfile.X,
															PhysicalProperty.Length,
															"Position On Profile Line");
				//set cubic interpolation for output
				daLineProfile.InterpolationMethodForEquidistantSampling = InterpolationMethod.Cubic6P;
				//show generated data array
				Globals.DataDisplay.ShowDocument(daLineProfile, "Line Profile of " + surfaceNameInUserDefinedInterfaceCatalog);
				#endregion
			}
			else {
				#region output 2D height profile
				//generate sampling parameter for 2D profile evaluation
				SamplingParameters sParaProfile = new SamplingParameters();
				sParaProfile.SamplingPoints = numberSamplingPointRectangleEvaluation;
				sParaProfile.SamplingDistance = sizeForRectangleEvaluation / numberSamplingPointRectangleEvaluation;

				//generate matrix to store height values
				ComplexField cfHeightValues = new ComplexField(sParaProfile.SamplingPoints, false, PrecisionMode.Double);

				//loop over all points
				for (int runY = 0; runY < sParaProfile.SamplingPoints.Y; runY++) {
					for (int runX = 0; runX < sParaProfile.SamplingPoints.X; runX++) {
						//convert current pixel coordinate to physical coordinate
						VectorD currCoordinate = CoordinateTransformations.PointFromPixelToPhysicalCoordinates(new Vector(runX, runY), sParaProfile);
						//apply coordinate shift
						currCoordinate += centerPositionRectangleEvaluation;
						//evaluate profile height at current position
						double profileHeight = pInterface.ProfileHeight(currCoordinate);
						//store evaluated profile height within matrix
						cfHeightValues[runX, runY] = profileHeight;
					}
				}

				//generate data array to display the height profile
				DataArray2D daHeightValues = new DataArray2D(new ComplexFieldArray(new ComplexField[] { cfHeightValues }),
															 new PhysicalProperty[] { PhysicalProperty.Length },
															 new string[] { "Height Values" },
															 sParaProfile.SamplingDistance.X,
															 (-0.5 * sParaProfile.Diameter.X) + (0.5 * sParaProfile.SamplingDistance.X) + centerPositionRectangleEvaluation.X,
															 PhysicalProperty.Length,
															 "X",
															 sParaProfile.SamplingDistance.Y,
															 (-0.5 * sParaProfile.Diameter.Y) + (0.5 * sParaProfile.SamplingDistance.Y) + centerPositionRectangleEvaluation.Y,
															 PhysicalProperty.Length,
															 "Y");
				//set cubic interpolation for output
				daHeightValues.InterpolationMethodForEquidistantSampling_X = InterpolationMethod.Cubic6P;
				daHeightValues.InterpolationMethodForEquidistantSampling_Y = InterpolationMethod.Cubic6P;
				//show generated data array
				Globals.DataDisplay.ShowDocument(daHeightValues, "2D Height Profile of " + surfaceNameInUserDefinedInterfaceCatalog);
				#endregion
			}

		}
	}
}
